#!/usr/bin/env python3

import tornado.httpserver
import tornado.websocket
import tornado.ioloop
from tornado.ioloop import PeriodicCallback
import tornado.web
import nnpy
import json
from nnpy import DONTWAIT
import threading
import os
import sys
import logging
from time import sleep
from multiprocessing.pool import ThreadPool

class Error(Exception) :
    pass

class WebSocketClosedError(Error) :
    pass

timeInterval= 1 #Milliseconds

class WSHandler(tornado.websocket.WebSocketHandler):
    # A Dictionary for storing the clients as a key and value corresponds to there connection status.
   

    def initialize(self, nanosoc_pub, nanosoc_sub):
        self.nanosoc_pub = nanosoc_pub
        self.nanosoc_sub = nanosoc_sub  
        self.clients_dict = {}      

    #check_origin fixes an error 403 with Tornado
    #http://stackoverflow.com/questions/24851207/tornado-403-get-warning-when-opening-websocket
    def check_origin(self, origin):
        return True

    def open(self):
        #Client Added with Connected Value, if already present then no entry will be there.
        self.clients_dict[self] = "Connected"

        #Send message periodic via socket upon a time interval        
        self.callback = PeriodicCallback(self.send_values, timeInterval)
        self.callback.start()

    def send_values(self):
        if not(isinstance(self.nanosoc_pub,int)):
            #Generates random values to send via websocket
            try:
                message = self.nanosoc_pub.recv(DONTWAIT)
                if len(message) > 0:
                    client_ip  = self.request.remote_ip     
                    client_port = self.request.connection.context.address
                    self.publish_to_clients(message)    
                else:
                    print("Msg not available")
            except:
                # print "Exception" 
               pass

    # This function will publish the signal value to all the connected clients.
    def publish_to_clients(self , y) :
        
        for i in self.clients_dict :
            try :
                client_port = i.request.connection.context.address   
                print("client_port="+str(client_port))             
                if(self.clients_dict[i] == "Connected") :                    
                    i.write_message(y)
            except :
                #If a exception is there than that client status will change to Disconnected.
                self.clients_dict[i] = "Disconnected"
                pass

    def on_message(self, message):
        if not(isinstance(self.nanosoc_sub,int)):
            print(message)
            self.nanosoc_sub.send(message)
            print("message sent")
        pass

    def on_close(self):
        self.callback.stop()


def websocpublish(self , ipcAddr, index, IPAddress, port, socType):
    #SocType 
    #1- Pub only
    #2- sub Only
    #3 -pubsub

    subSock =  -1;
    pubSock = -1;

    if ((socType == 1) or (socType == 3)):
        ipcAddr = 'ipc:///tmp/' + modelname + '_' +'websocPub'+ index +'.ipc'
        pubSock = nnpy.Socket(nnpy.AF_SP,nnpy.SUB)
        
        ret1 = pubSock.connect(ipcAddr)
        ret2 = pubSock.setsockopt(nnpy.SUB, nnpy.SUB_SUBSCRIBE, '')
        
    if ((socType == 2) or (socType == 3)):
        ipcAddr = 'ipc:///tmp/' + modelname + '_' +'websocSub'+ index +'.ipc'        
        subSock = nnpy.Socket(nnpy.AF_SP, nnpy.PUB)
        
        ret1 = subSock.bind(ipcAddr)
    IPAddr = IPAddress
    webPort = int(port)

    import asyncio
    asyncio.set_event_loop(asyncio.new_event_loop())
    httpwebserver = tornado.httpserver.HTTPServer(tornado.web.Application([(r'/', WSHandler, {'nanosoc_pub':pubSock, 'nanosoc_sub':subSock}),]))
    
    httpwebserver.listen(webPort , IPAddr)
    tornado.ioloop.IOLoop.instance().start()

class webpublish_constructor:
    
    def __init__(self):
        self.allClasses = []
        

    def construct(self,ipcName,index, IPAddress, port, socType, ):

        
        instance = threading.Thread(target=websocpublish,args=(self , ipcName,index, IPAddress, port,socType, )) 
        instance.start()
        self.allClasses.append(instance)



if __name__ == "__main__":

    if (len(sys.argv) != 2):
        raise Exception ("Incorrect Number of Inputs. Provide the path to the MW_websocInfo.json file")

    jsonFileName = sys.argv[1]
    jsonFileHandle = open(jsonFileName)
    jsonData = json.load(jsonFileHandle)
    modelname = jsonData["ModelName"]
    numBlocks = int(len(jsonData["Servers"]))   

    
    webpublish_constructor_obj = webpublish_constructor()

    for i in range(len(jsonData["Servers"])):
        webpublish_constructor_obj.construct(modelname, str(i+1), jsonData["Servers"][i]["IPAddress"], jsonData["Servers"][i]["Port"], (jsonData["Servers"][i]["PubBlocks"] + 2*jsonData["Servers"][i]["SubBlocks"]))  
   
    while 1:
       sleep(0.5)
       pass